/*
 * cambalache.css
 *
 * Copyright (C) 2021-2024 Juan Pablo Ugarte
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Author: Juan Pablo Ugarte <juanpablougarte@gmail.com>
 *
 */

window.cmb-window label.message {
  padding: 1ex 1em;
  border-radius: 1ex 1ex 0 0;
  color: white;
  background-color: rgba(0, 0, 0, .6);
}

window.cmb-window.dark .message {
  color: black;
  background-color: rgba(255, 255, 255, .6);
}

window.cmb-window stackswitcher.compact > button {
  min-width: unset;
}

window.cmb-window box.donate {
  padding: 1em;
}

popover.cmb-tutor > * {
  padding: 1em;
}

popover.cmb-tutor label {
  font-size: 18px;
  font-weight: bold;
}

popover.cmb-tutor image {
  padding-right: 1em;
  -gtk-icon-size: 48px;
}

button.cmb-tutor-highlight,
modelbutton.cmb-tutor-highlight,
buttonbox.cmb-tutor-highlight > button,
menubutton.cmb-tutor-highlight > button,
stackswitcher.cmb-tutor-highlight > button,
stack.cmb-tutor-highlight,
entry.cmb-tutor-highlight,
treeview.cmb-tutor-highlight,
box.cmb-tutor-highlight,
CmbView.cmb-tutor-highlight {
  box-shadow: inset 0px 0px 6px @theme_selected_bg_color;
  transition: box-shadow .75s ease;
}

CmbView.cmb-tutor-highlight {
  padding: 6px;
}

CmbTypeChooser {
  padding: 4px;
}

CmbUIPropertyEditor,
CmbFragmentEditor,
CmbObjectPropertyEditor,
CmbAccessibleEditor {
  padding: 0 4px 4px 4px;
}

CmbCSSEditor,
CmbGResourceEditor,
CmbGResourceFileEditor,
stackswitcher.property-pane {
  padding: 4px;
}

CmbTextView > textview,
CmbObjectListEditor > textview {
  border: solid @borders 1px;
}

button.borderless {
 border: unset;
}

image.icon-size-32 {
  -gtk-icon-size: 32px;
}

image.icon-size-64 {
  -gtk-icon-size: 64px;
}

windowtitle.changed {
  font-style: italic;
}

CmbDonateDialog label.donate-desc {
  font-size: 1.1em;
}

popover.notifications > * {
  background-color: @window_bg_color;
}

/* Help window */
window.cmb-help box.docs {
  margin: 1em 3em;
}

window.cmb-help box.docs > picture:first-child {
  margin-bottom:  2em;
}

window.cmb-help box.docs > picture {
  margin:  1em 0;
}

window.cmb-help label.title-1 {
  margin-bottom: 1ex;
}

window.cmb-help label.title-2,
window.cmb-help label.title-3,
window.cmb-help label.title-4 {
  margin-top: 1em;
  margin-bottom: 1ex;
}

window.cmb-help label.message {
  margin-top: 1em;
  margin-bottom: 1ex;
  padding: 1ex 1em;
  border-radius: 1ex;
  color: white;
  background-color: rgba(0, 0, 0, .6);
}

window.cmb-help.dark .message {
  color: black;
  background-color: rgba(255, 255, 255, .6);
}

window.cmb-help .code {
  margin: 1em 0;
  padding: 1ex 1em;
  border-radius: 1ex;
  border: 1px solid @theme_selected_bg_color;
  font-family: monospace;
}
