/*
 * Merengue CSS
 *
 * Copyright (C) 2021  Juan Pablo Ugarte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors:
 *   Juan Pablo Ugarte <juanpablougarte@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-only
 *
 */

.merengue_selected:not(.gtk4):not(.gtk3) {
  box-shadow: inset 0px 0px 4px 2px @theme_selected_bg_color;
}

.gtk4,
.gtk3 > decoration {
  box-shadow: 0 3px 9px 1px rgba(0, 0, 0, 0.5), 0 0 0 1px rgba(0, 0, 0, 0.23);
}

.gtk4.merengue_selected,
.gtk3.merengue_selected > decoration {
  box-shadow: 0 3px 9px 1px @theme_selected_bg_color, 0 0 0 1px rgba(0, 0, 0, 0.23);
}

MrgPlaceholder {
  background: url('resource:///ar/xjuan/Merengue/placeholder.svg');
  margin: 2px;
  border-bottom: solid 1px gray;
  border-right: solid 1px gray;
}

